//---------------------------------------------------------------------------
#ifndef StringsH
#define StringsH

#include <stdio.h>
//---------------------------------------------------------------------------

#define SEPARATE_TABLE_SIZE  128

class StringSeparator
{
public:
	enum CharacterCode {
		CharCode_Ansi,
		CharCode_Utf8,
		CharCode_Wide,
	};
	enum SeparatorTableValue {
		Separator_Normal,
		Separator_Symbol,
		Separator_Delimiter,
	};
protected:
	CharacterCode mCharCode;

	char **scan;
	int MaxScan, CurCount;
	char SeparateTable[SEPARATE_TABLE_SIZE];
	char StringChar;

	bool SetScan(int index, const char *str);
	bool SetScan(int index, const wchar_t *wstr);
private:
	char *mSeparateBuffer;
	size_t mSeparateBufferSize;
public:
	StringSeparator();
	virtual ~StringSeparator();

	CharacterCode GetCharacterCode() const;
	void SetCharacterCode(CharacterCode code);

	void GetSeparateTable(char *table) const;
	void SetSeparateTable(const char *table);
	char GetSeparateTableValue(int index) const;
	void SetSeparateTableValue(int index, char value);
	char GetStringChar(void) const;
	void SetStringChar(char ch);

	int Separate(const char *str);

	bool CheckCommand(const char *comstr, int optc);
	bool CheckString(int index, const char *str);

	bool chkcom(const char *comstr, int optc) 	{ return CheckCommand(comstr, optc); }
	bool chkstr(int index, const char *str) 	{ return CheckString(index, str); }

	int ToInt(int index) const;
	int ToHex(int index) const;
	double ToDouble(int index) const;
	float ToFloat(int index)  const		{ return (float)ToDouble(index); }
	const char *GetString(int index) const 	{ return scan[index]; }

	int GetCount(void) const 			{ return CurCount; }
};


class LineStringSeparator : public StringSeparator
{
public:
	LineStringSeparator();
	~LineStringSeparator();

	const char *GetRawLine();
protected:
	char *mLineBuffer;
	size_t mLineBufferSize;
	char *mRawLine;
};


class FileStringSeparator : public LineStringSeparator
{
protected:
	FILE *fh;
public:
	FileStringSeparator(FILE *_fh);
	~FileStringSeparator();

	virtual int GetLine(void);
	inline FILE *GetFileHandle(void) {return fh;}
};

class AsciiFileString : public FileStringSeparator
{
public:
	AsciiFileString(FILE *_fh);
	virtual int GetLine(void) override;
private:
	int mLineCount;
};

class MacFileString : public FileStringSeparator
{
public:
	MacFileString(FILE *_fh);
	virtual int GetLine(void);
};
//---------------------------------------------------------------------------
int GetStrings(char *src, char *dest[], int maxdest);
//---------------------------------------------------------------------------
#endif
